@@ -1,7 +1,12 @@
 import asyncio
 import logging
+from mock import Mock
 import re
 import ssl
+import sys
+import tqdm
+import time
+from typing import Callable, Any, Iterable, Tuple
 
 import aiohttp
 import tqdm.asyncio
@@ -21,6 +26,7 @@ supported_recursive_search_ids = (
     'ok_id',
     'wikimapia_uid',
     'steam_id',
+    'uidme_uguid',
 )
 
 common_errors = {
@@ -36,6 +42,103 @@ common_errors = {
 
 unsupported_characters = '#'
 
+QueryDraft = Tuple[Callable, Any, Any]
+QueriesDraft = Iterable[QueryDraft]
+
+
+class AsyncExecutor:
+    def __init__(self, *args, **kwargs):
+        self.logger = kwargs['logger']
+
+    async def run(self, tasks: QueriesDraft):
+        start_time = time.time()
+        results = await self._run(tasks)
+        self.execution_time = time.time() - start_time
+        self.logger.debug(f'Spent time: {self.execution_time}')
+        return results
+
+    async def _run(self, tasks: QueriesDraft):
+        await asyncio.sleep(0)
+
+
+class AsyncioSimpleExecutor(AsyncExecutor):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    async def _run(self, tasks: QueriesDraft):
+        futures = [f(*args, **kwargs) for f, args, kwargs in tasks]
+        return await asyncio.gather(*futures)
+
+
+class AsyncioProgressbarExecutor(AsyncExecutor):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    async def _run(self, tasks: QueriesDraft):
+        futures = [f(*args, **kwargs) for f, args, kwargs in tasks]
+        results = []
+        for f in tqdm.asyncio.tqdm.as_completed(futures):
+            results.append(await f)
+        return results
+
+
+class AsyncioProgressbarSemaphoreExecutor(AsyncExecutor):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.semaphore = asyncio.Semaphore(kwargs.get('in_parallel', 1))
+
+    async def _run(self, tasks: QueriesDraft):
+        async def _wrap_query(q: QueryDraft):
+            async with self.semaphore:
+                f, args, kwargs = q
+                return await f(*args, **kwargs)
+
+        async def semaphore_gather(tasks: QueriesDraft):
+            coros = [_wrap_query(q) for q in tasks]
+            results = []
+            for f in tqdm.asyncio.tqdm.as_completed(coros):
+                results.append(await f)
+            return results
+
+        return await semaphore_gather(tasks)
+
+
+class AsyncioProgressbarQueueExecutor(AsyncExecutor):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.workers_count = kwargs.get('in_parallel', 10)
+        self.progress_func = kwargs.get('progress_func', tqdm.tqdm)
+        self.queue = asyncio.Queue(self.workers_count)
+
+    async def worker(self):
+        while True:
+            f, args, kwargs = await self.queue.get()
+            result = await f(*args, **kwargs)
+            self.results.append(result)
+            self.progress.update(1)
+            self.queue.task_done()
+
+    async def _run(self, tasks: QueriesDraft):
+        self.results = []
+
+        if sys.version_info.minor > 6:
+            create_task = asyncio.create_task
+        else:
+            loop = asyncio.get_event_loop()
+            create_task = loop.create_task
+
+        workers = [create_task(self.worker())
+                   for _ in range(self.workers_count)]
+        task_list = list(tasks)
+        self.progress = self.progress_func(total=len(task_list))
+        for t in task_list:
+            await self.queue.put(t)
+        await self.queue.join()
+        for w in workers:
+            w.cancel()
+        self.progress.close()
+        return self.results
+
 
 async def get_response(request_future, site_name, logger):
     html_text = None
@@ -61,9 +164,6 @@ async def get_response(request_future, site_name, logger):
     except asyncio.TimeoutError as errt:
         error_text = "Timeout Error"
         expection_text = str(errt)
-    except (ssl.SSLCertVerificationError, ssl.SSLError) as err:
-        error_text = "SSL Error"
-        expection_text = str(err)
     except aiohttp.client_exceptions.ClientConnectorError as err:
         error_text = "Error Connecting"
         expection_text = str(err)
@@ -74,28 +174,33 @@ async def get_response(request_future, site_name, logger):
         error_text = "Proxy Error"
         expection_text = str(err)
     except Exception as err:
-        logger.warning(f'Unhandled error while requesting {site_name}: {err}')
-        logger.debug(err, exc_info=True)
-        error_text = "Some Error"
-        expection_text = str(err)
+        # python-specific exceptions
+        if sys.version_info.minor > 6:
+            if isinstance(err, ssl.SSLCertVerificationError) or isinstance(err, ssl.SSLError):
+                error_text = "SSL Error"
+                expection_text = str(err)
+        else:
+            logger.warning(f'Unhandled error while requesting {site_name}: {err}')
+            logger.debug(err, exc_info=True)
+            error_text = "Some Error"
+            expection_text = str(err)
 
     # TODO: return only needed information
     return html_text, status_code, error_text, expection_text
 
 
-async def update_site_dict_from_response(sitename, site_dict, results_info, semaphore, logger, query_notify):
-    async with semaphore:
-        site_obj = site_dict[sitename]
-        future = site_obj.request_future
-        if not future:
-            # ignore: search by incompatible id type
-            return
+async def update_site_dict_from_response(sitename, site_dict, results_info, logger, query_notify):
+    site_obj = site_dict[sitename]
+    future = site_obj.request_future
+    if not future:
+        # ignore: search by incompatible id type
+        return
 
-        response = await get_response(request_future=future,
-                                      site_name=sitename,
-                                      logger=logger)
+    response = await get_response(request_future=future,
+                                  site_name=sitename,
+                                  logger=logger)
 
-        site_dict[sitename] = process_site_result(response, query_notify, logger, results_info, site_obj)
+    return sitename, process_site_result(response, query_notify, logger, results_info, site_obj)
 
 
 # TODO: move to separate class
@@ -162,7 +267,7 @@ def process_site_result(response, query_notify, logger, results_info, site: Maig
 
     if status_code and not error_text:
         error_text, site_error_text = detect_error_page(html_text, status_code, failure_errors,
-                                                        site.ignore_403)
+                                                        site.ignore403)
 
     if site.activation and html_text:
         is_need_activation = any([s for s in site.activation['marks'] if s in html_text])
@@ -174,6 +279,8 @@ def process_site_result(response, query_notify, logger, results_info, site: Maig
                 activate_fun(site, logger)
             except AttributeError:
                 logger.warning(f'Activation method {method} for site {site.name} not found!')
+            except Exception as e:
+                logger.warning(f'Failed activation {method} for site {site.name}: {e}')
 
     # presense flags
     # True by default
@@ -288,26 +395,32 @@ def process_site_result(response, query_notify, logger, results_info, site: Maig
     return results_info
 
 
-async def maigret(username, site_dict, query_notify, logger,
-                  proxy=None, timeout=None, recursive_search=False,
+async def maigret(username, site_dict, logger, query_notify=None,
+                  proxy=None, timeout=None, is_parsing_enabled=False,
                   id_type='username', debug=False, forced=False,
                   max_connections=100, no_progressbar=False,
                   cookies=None):
     """Main search func
 
-    Checks for existence of username on various social media sites.
+    Checks for existence of username on certain sites.
 
     Keyword Arguments:
-    username               -- String indicating username that report
-                              should be created against.
-    site_dict              -- Dictionary containing all of the site data.
+    username               -- Username string will be used for search.
+    site_dict              -- Dictionary containing sites data.
     query_notify           -- Object with base type of QueryNotify().
                               This will be used to notify the caller about
                               query results.
-    proxy                  -- String indicating the proxy URL
+    logger                 -- Standard Python logger object.
     timeout                -- Time in seconds to wait before timing out request.
                               Default is no timeout.
-    recursive_search       -- Search for other usernames in website pages & recursive search by them.
+    is_parsing_enabled     -- Extract additional info from account pages.
+    id_type                -- Type of username to search.
+                              Default is 'username', see all supported here:
+                              https://github.com/soxoj/maigret/wiki/Supported-identifier-types
+    max_connections        -- Maximum number of concurrent connections allowed.
+                              Default is 100.
+    no_progressbar         -- Displaying of ASCII progressbar during scanner.
+    cookies                -- Filename of a cookie jar file to use for each request.
 
     Return Value:
     Dictionary containing results from report. Key of dictionary is the name
@@ -324,6 +437,9 @@ async def maigret(username, site_dict, query_notify, logger,
     """
 
     # Notify caller that we are starting the query.
+    if not query_notify:
+        query_notify = Mock()
+
     query_notify.start(username, id_type)
 
     # TODO: connector
@@ -364,7 +480,7 @@ async def maigret(username, site_dict, query_notify, logger,
 
         # Record URL of main site and username
         results_site['username'] = username
-        results_site['parsing_enabled'] = recursive_search
+        results_site['parsing_enabled'] = is_parsing_enabled
         results_site['url_main'] = site.url_main
         results_site['cookies'] = cookie_jar and cookie_jar.filter_cookies(site.url_main) or None
 
@@ -374,7 +490,7 @@ async def maigret(username, site_dict, query_notify, logger,
 
         headers.update(site.headers)
 
-        if not 'url' in site.__dict__:
+        if 'url' not in site.__dict__:
             logger.error('No URL for site %s', site.name)
         # URL of user on site (if it exists)
         url = site.url.format(
@@ -448,28 +564,33 @@ async def maigret(username, site_dict, query_notify, logger,
         # Add this site's results into final dictionary with all of the other results.
         results_total[site_name] = results_site
 
-    # TODO: move into top-level function
-
-    sem = asyncio.Semaphore(max_connections)
-
-    tasks = []
+    coroutines = []
     for sitename, result_obj in results_total.items():
-        update_site_coro = update_site_dict_from_response(sitename, site_dict, result_obj, sem, logger, query_notify)
-        future = asyncio.ensure_future(update_site_coro)
-        tasks.append(future)
+        coroutines.append((update_site_dict_from_response, [sitename, site_dict, result_obj, logger, query_notify], {}))
 
     if no_progressbar:
-        await asyncio.gather(*tasks)
+        executor = AsyncioSimpleExecutor(logger=logger)
     else:
-        for f in tqdm.asyncio.tqdm.as_completed(tasks):
-            await f
+        executor = AsyncioProgressbarQueueExecutor(logger=logger, in_parallel=max_connections, timeout=timeout+0.5)
+
+    results = await executor.run(coroutines)
 
     await session.close()
 
     # Notify caller that all queries are finished.
     query_notify.finish()
 
-    return results_total
+    data = {}
+    for result in results:
+        # TODO: still can be empty
+        if result:
+            try:
+                data[result[0]] = result[1]
+            except Exception as e:
+                logger.error(e, exc_info=True)
+                logger.info(result)
+
+    return data
 
 
 def timeout_check(value):
@@ -498,7 +619,6 @@ def timeout_check(value):
 
 
 async def site_self_check(site, logger, semaphore, db: MaigretDatabase, silent=False):
-    query_notify = Mock()
     changes = {
         'disabled': False,
     }
@@ -518,10 +638,9 @@ async def site_self_check(site, logger, semaphore, db: MaigretDatabase, silent=F
     for username, status in check_data:
         async with semaphore:
             results_dict = await maigret(
-                username,
-                {site.name: site},
-                query_notify,
-                logger,
+                username=username,
+                site_dict={site.name: site},
+                logger=logger,
                 timeout=30,
                 id_type=site.type,
                 forced=True,
