@@ -1,23 +1,27 @@
 """
 Maigret main module
 """
-
+import aiohttp
+import asyncio
+import logging
 import os
-import platform
 import sys
+import platform
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
 
 import requests
-from socid_extractor import parse, __version__ as socid_version
+from socid_extractor import extract, parse, __version__ as socid_version
 
-from .checking import *
+from .checking import timeout_check, supported_recursive_search_ids, self_check, unsupported_characters, maigret
 from .notify import QueryNotifyPrint
 from .report import save_csv_report, save_xmind_report, save_html_report, save_pdf_report, \
     generate_report_context, save_txt_report, SUPPORTED_JSON_REPORT_FORMATS, check_supported_json_format, \
     save_json_report
+from .sites import MaigretDatabase
 from .submit import submit_dialog
+from .utils import get_dict_ascii_tree
 
-__version__ = '0.1.14'
+__version__ = '0.1.17'
 
 
 async def main():
@@ -106,7 +110,11 @@ async def main():
                         )
     parser.add_argument("--no-recursion",
                         action="store_true", dest="disable_recursive_search", default=False,
-                        help="Disable parsing pages for other usernames and recursive search by them."
+                        help="Disable recursive search by additional data extracted from pages."
+                        )
+    parser.add_argument("--no-extracting",
+                        action="store_true", dest="disable_extracting", default=False,
+                        help="Disable parsing pages for additional data and other usernames."
                         )
     parser.add_argument("--self-check",
                         action="store_true", default=False,
@@ -172,7 +180,7 @@ async def main():
                         action="store", metavar='REPORT_TYPE',
                         dest="json", default='', type=check_supported_json_format,
                         help=f"Generate a JSON report of specific type: {', '.join(SUPPORTED_JSON_REPORT_FORMATS)}"
-                        " (one report per username)."
+                             " (one report per username)."
                         )
 
     args = parser.parse_args()
@@ -200,9 +208,10 @@ async def main():
         u: args.id_type
         for u in args.username
         if u not in ['-']
-        and u not in args.ignore_ids_list
+           and u not in args.ignore_ids_list
     }
 
+    parsing_enabled = not args.disable_extracting
     recursive_search_enabled = not args.disable_recursive_search
 
     # Make prompts
@@ -210,15 +219,29 @@ async def main():
         print("Using the proxy: " + args.proxy)
 
     if args.parse_url:
-        page, _ = parse(args.parse_url, cookies_str='')
-        info = extract(page)
-        text = 'Extracted ID data from webpage: ' + ', '.join([f'{a}: {b}' for a, b in info.items()])
-        print(text)
-        for k, v in info.items():
-            if 'username' in k:
-                usernames[v] = 'username'
-            if k in supported_recursive_search_ids:
-                usernames[v] = k
+        # url, headers
+        reqs = [(args.parse_url, set())]
+        try:
+            # temporary workaround for URL mutations MVP
+            from socid_extractor import mutate_url
+            reqs += list(mutate_url(args.parse_url))
+        except:
+            pass
+
+        for req in reqs:
+            url, headers = req
+            print(f'Scanning webpage by URL {url}...')
+            page, _ = parse(url, cookies_str='', headers=headers)
+            info = extract(page)
+            if not info:
+                print('Nothing extracted')
+            else:
+                print(get_dict_ascii_tree(info.items(), new_line=False), ' ')
+            for k, v in info.items():
+                if 'username' in k:
+                    usernames[v] = 'username'
+                if k in supported_recursive_search_ids:
+                    usernames[v] = k
 
     if args.tags:
         args.tags = list(set(str(args.tags).split(',')))
@@ -248,7 +271,7 @@ async def main():
     site_data = get_top_sites_for_id(args.id_type)
 
     if args.new_site_to_submit:
-        is_submitted = await submit_dialog(db, args.new_site_to_submit)
+        is_submitted = await submit_dialog(db, args.new_site_to_submit, args.cookie_file)
         if is_submitted:
             db.save_to_file(args.db_file)
 
@@ -257,7 +280,7 @@ async def main():
         print('Maigret sites database self-checking...')
         is_need_update = await self_check(db, site_data, logger, max_connections=args.connections)
         if is_need_update:
-            if input('Do you want to save changes permanently? [yYnN]\n').lower() == 'y':
+            if input('Do you want to save changes permanently? [Yn]\n').lower() == 'y':
                 db.save_to_file(args.db_file)
                 print('Database was successfully updated.')
             else:
@@ -269,7 +292,6 @@ async def main():
 
     # Make reports folder is not exists
     os.makedirs(args.folderoutput, exist_ok=True)
-    report_path = args.folderoutput
 
     # Define one report filename template
     report_filepath_tpl = os.path.join(args.folderoutput, 'report_{username}{postfix}')
@@ -319,12 +341,12 @@ async def main():
 
         sites_to_check = get_top_sites_for_id(id_type)
 
-        results = await maigret(username,
-                                dict(sites_to_check),
-                                query_notify,
+        results = await maigret(username=username,
+                                site_dict=dict(sites_to_check),
+                                query_notify=query_notify,
                                 proxy=args.proxy,
                                 timeout=args.timeout,
-                                recursive_search=recursive_search_enabled,
+                                is_parsing_enabled=parsing_enabled,
                                 id_type=id_type,
                                 debug=args.verbose,
                                 logger=logger,
@@ -333,14 +355,13 @@ async def main():
                                 max_connections=args.connections,
                                 )
 
-        username_result = (username, id_type, results)
         general_results.append((username, id_type, results))
 
         # TODO: tests
         for website_name in results:
             dictionary = results[website_name]
             # TODO: fix no site data issue
-            if not dictionary:
+            if not dictionary or not recursive_search_enabled:
                 continue
 
             new_usernames = dictionary.get('ids_usernames')
@@ -375,7 +396,6 @@ async def main():
             save_json_report(filename, username, results, report_type=args.json)
             query_notify.warning(f'JSON {args.json} report for {username} saved in {filename}')
 
-
     # reporting for all the result
     if general_results:
         if args.html or args.pdf:
